package com.loafle.overflow.model.infra;

import java.util.List;

import javax.persistence.*;

import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import com.loafle.overflow.core.json.InfraHostIPsSerializer;
import com.loafle.overflow.model.meta.MetaTargetHostType;

import org.hibernate.annotations.Fetch;
import org.hibernate.annotations.FetchMode;

/**
 * Created by root on 17. 6. 22.
 */
@Entity
@Table(name = "INFRA_HOST", schema = "public")
@DiscriminatorValue("2")
public class InfraHost extends Infra {
  private InfraZone infraZone;
  private MetaTargetHostType metaTargetHostType;

  private InfraHostMachine infraHostMachine;
  private InfraHostOS infraHostOS;

  @JsonSerialize(using = InfraHostIPsSerializer.class)
  private List<InfraHostIP> infraHostIPs;

  private InfraHost containerInfraHost;

  public InfraHost() {
    super();
  }

  public InfraHost(Long id) {
    super(id);
  }

  @ManyToOne
  @JoinColumn(name = "INFRA_ZONE_ID", nullable = false)
  public InfraZone getInfraZone() {
    return infraZone;
  }

  public void setInfraZone(InfraZone infraZone) {
    this.infraZone = infraZone;
  }

  @ManyToOne
  @JoinColumn(name = "META_TARGET_HOST_TYPE_ID", nullable = false)
  public MetaTargetHostType getMetaTargetHostType() {
    return metaTargetHostType;
  }

  public void setMetaTargetHostType(MetaTargetHostType metaTargetHostType) {
    this.metaTargetHostType = metaTargetHostType;
  }

  /**
   * @return the infraHostMachine
   */
  @OneToOne(fetch = FetchType.EAGER, cascade = CascadeType.ALL)
  @JoinColumn(name = "INFRA_HOST_MACHINE_ID", nullable = true)
  public InfraHostMachine getInfraHostMachine() {
    return infraHostMachine;
  }

  /**
   * @param infraHostMachine the infraHostMachine to set
   */
  public void setInfraHostMachine(InfraHostMachine infraHostMachine) {
    this.infraHostMachine = infraHostMachine;
  }

  /**
   * @return the infraHostOS
   */
  @OneToOne(fetch = FetchType.EAGER, cascade = CascadeType.ALL)
  @JoinColumn(name = "INFRA_HOST_OS_ID", nullable = true)
  public InfraHostOS getInfraHostOS() {
    return infraHostOS;
  }

  /**
   * @param infraHostOS the infraHostOS to set
   */
  public void setInfraHostOS(InfraHostOS infraHostOS) {
    this.infraHostOS = infraHostOS;
  }

  /**
   * @return the infraHostIPs
   */
  @OneToMany(fetch = FetchType.EAGER, cascade = CascadeType.ALL, orphanRemoval = true, mappedBy = "infraHost")
  @Fetch(FetchMode.SELECT)
  public List<InfraHostIP> getInfraHostIPs() {
    return infraHostIPs;
  }

  /**
   * @param infraHostIPs the infraHostIPs to set
   */
  public void setInfraHostIPs(List<InfraHostIP> infraHostIPs) {
    this.infraHostIPs = infraHostIPs;
  }

  /**
   * @return the containerInfraHost
   */
  @ManyToOne
  @JoinColumn(name = "CONTAINER_INFRA_HOST_ID", nullable = true)
  public InfraHost getContainerInfraHost() {
    return containerInfraHost;
  }

  /**
   * @param containerInfraHost the containerInfraHost to set
   */
  public void setContainerInfraHost(InfraHost containerInfraHost) {
    this.containerInfraHost = containerInfraHost;
  }

}
